@@ -9,6 +9,7 @@ import (
 	"time"
 
 	"github.com/AdguardTeam/golibs/errors"
+	"github.com/AdguardTeam/golibs/timeutil"
 )
 
 // macKey contains hardware address as byte array of 6, 8, or 20 bytes.
@@ -117,6 +118,22 @@ func (iface *netInterface) removeLease(l *Lease) (err error) {
 	return nil
 }
 
+// blockLease marks l as blocked for a configured TTL, as reported by
+// [Lease.IsBlocked].  It also removes the lease from iface.  l must not be nil.
+func (iface *netInterface) blockLease(l *Lease, clock timeutil.Clock) {
+	mk := macToKey(l.HWAddr)
+
+	iface.indexMu.Lock()
+	defer iface.indexMu.Unlock()
+
+	l.HWAddr = blockedHardwareAddr
+	l.Hostname = ""
+	l.Expiry = clock.Now().Add(iface.leaseTTL)
+	l.IsStatic = false
+
+	delete(iface.leases, mk)
+}
+
 // nextIP generates a new free IP.
 func (iface *netInterface) nextIP() (ip netip.Addr) {
 	r := iface.addrSpace
