@@ -126,6 +126,27 @@ func (l *queryLog) search(
 		entries = entries[:totalLimit]
 	}
 
+	entries, oldest = l.finalizeSearchResults(entries, params, oldest)
+
+	l.logger.DebugContext(
+		ctx,
+		"prepared data",
+		"count", len(entries),
+		"total", total,
+		"older_than", params.olderThan,
+		"elapsed", time.Since(start),
+	)
+
+	return entries, oldest
+}
+
+// finalizeSearchResults sorts entries and applies offset trimming, and updates
+// the oldest timestamp.  params must not be nil.
+func (l *queryLog) finalizeSearchResults(
+	entries []*logEntry,
+	params *searchParams,
+	oldest time.Time,
+) (res []*logEntry, t time.Time) {
 	// Resort entries on start time to partially mitigate query log looking
 	// weird on the frontend.
 	//
@@ -138,8 +159,7 @@ func (l *queryLog) search(
 		if len(entries) > params.offset {
 			entries = entries[params.offset:]
 		} else {
-			entries = make([]*logEntry, 0)
-			oldest = time.Time{}
+			return nil, time.Time{}
 		}
 	}
 
@@ -148,15 +168,6 @@ func (l *queryLog) search(
 		oldest = entries[len(entries)-1].Time
 	}
 
-	l.logger.DebugContext(
-		ctx,
-		"prepared data",
-		"count", len(entries),
-		"total", total,
-		"older_than", params.olderThan,
-		"elapsed", time.Since(start),
-	)
-
 	return entries, oldest
 }
 
